# Devilbox Documentation

[Overview](README.md) |
[Quickstart](Quickstart.md) |
[Install](Install.md) |
[Update](Update.md) |
Configure |
[Run](Run.md) |
[Usage](Usage.md) |
[OS](OS.md) |
[Backups](Backups.md) |
[Examples](Examples.md) |
[Technical](Technical.md) |
[Hacking](Hacking.md) |
[FAQ](FAQ.md)

---

## Configure

1. [Overview]()
2. [Devilbox general settings](#1-devilbox-general-settings)
  1. [Verbosity]()
  2. [Devilbox base path]()
  3. [Host computer listening address]()
3. [Project settings]()
  1. [Project domain]()
  2. [Project path]()
4. [Container settings]()
  1. [General]()
    1. [Timezone]()
    2. [User id]()
	3. [Group id]()
  2. [PHP / HHVM]()
	1. [Xdebug]()
	2. [php.ini]()
	3. [HHVM]()
  3. [Webserver]()
    1. [Host port]()
  4. [MySQL]()
    1. [Root password]()
	2. [General Log]()
	3. [Host port]()
    4. [Data path]()
    5. [my.cnf]()
  5. [PostgreSQL]()
    1. [Root user]()
	2. [Root password]()
	3. [Host port]()
    4. [Data path]()
  6. [Redis]()
    1. [Host port]()
  7. [Memcached]()
    1. [Host port]()
  8. [MongoDB]()
    1. [Host port]()
    2. [Data path]()
  9. [Bind]()
    1. [Upstream resolver]()
    2. [Host port]()
5. [Intranet settings]()
  1. [DNS check timeout]()
6. [Host computer]()
  1. [Auto-DNS]()
  2. [/etc/hosts/]()

---

## 1. Overview

## 2. Devilbox general settings

#### 2.1 Verbosity

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| DEBUG_COMPOSE_ENTRYPOINT  | `0`     | Set it to `1` or `0` |

If set to `1`, it will show all executed commands during docker startup.

#### 2.2 Devilbox base path

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| DEVILBOX_PATH             | `.`     | Relative or absolute path allowed |

This is the base path that will be prepended to all mount paths specified in `.env`.
You will usually not need to change this value..

#### 2.3 Host computer listening address

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| LOCAL_LISTEN_ADDRESS      | `127.0.0.1:`   | Address for every service to listen on the Docker host.<br/>Pay attention to the **`:`** after the IP address |

This determines the Host address your webserver and all other daemons should listen on.

In case it is not `127.0.0.1` (because you are using a VirtualBox Docker setup) change it to the IP address of the VirtualBox host. Otherwise you will not need to change this value.

1. When you remove it completely, it will listen on all interfaces.
2. When you use specific address, you must add a **`:`** at the end.

You can for example change it to `0.0.0.0:` or make it empty in order to listen on all interfaces. This enables it for other people inside your network to access the devilbox. Or you can even install the devilbox on a different computer and access it remotely.

## 3. Project settings

#### 3.1 Project domain

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| TLD_SUFFIX                | `loc`   | Domain suffix for all your project. Can also be a subdomain such `work.loc` |

Each project will be served by `http://<project-folder>.<TLD_SUFFIX>`. If you want to change the default `loc` domain suffix to something else such as local, adjust this variable. Here are a few examples:

| Project Folder | TLD_SUFFIX | URL |
|----------------|------------|-----|
| my-cake        | loc        | http://my-cake.loc      |
| my-cake        | local      | http://my-cake.local    |
| my-cake        | dev        | http://my-cake.dev      |
| my-cake        | work.loc   | http://my-cake.work.loc |
| test           | foo        | http://test.foo         |
| project1       | bar        | http://project1.bar     |

The above examples should make it clear enough.

#### 3.2 Project path

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PATH_HTTPD_DATADIR   | `./data/www`   | Can be absolute or relative path. A relative path starts inside the devilbox git directory. |

This is the file system path on your host computer which will hold the Project Folders.


## 4. Container settings

#### 4.1 General

##### 4.1.1 Timezone

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| TIMEZONE                  | `Europe/Berlin`| Set timezone of the Docker container |

Use this variable to control the timezone of the container. 

**Note:** This currently only works on the devilbox self-provided container:

* DNS
* MySQL/MariaDB
* PHP/HHVM
* Apache/Nginx

The official container do not support this setting.

##### 4.1.1 User id

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| NEW_UID                   | `1000`| Change the Docker containers user id (uid) |

This setting is used to change the user id of the PHP/HHVM container to the one you specify. As you will work insisde the PHP/HHVM container, you will have to set it to the same user id of your Host computers user id. It will make sure that files created inside the container have the same access permissions as outside the container.

To find out your user id, type the following on your Host system:
```shell
$ id
```

**Note:** If your Host computers user id and the containers user id do not match, files will have different access rights inside and outside which might result in permission errors like `access denied`. So make sure to set this value.

**Note:** Files created by the webserver such as uploads, tmp and cache files are still created by the webservers user id and you will probably have to `chmod` them. This issues will be addressed shortly and you will also be able to change the uid/gid of the webserver in the next devilbox release.

##### 4.1.2 Group id

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| NEW_GID                   | `1000`| Change the Docker containers group id (gid) |

This setting is used to change the group id of the PHP/HHVM container to the one you specify. As you will work insisde the PHP/HHVM container, you will have to set it to the same group id of your Host computers group id. It will make sure that files created inside the container have the same access permissions as outside the container.

To find out your group id, type the following on your Host system:
```shell
$ id
```

**Note:** If your Host computers group id and the containers group id do not match, files will have different access rights inside and outside which might result in permission errors like `access denied`. So make sure to set this value.

**Note:** Files created by the webserver such as uploads, tmp and cache files are still created by the webservers group id and you will probably have to `chmod` them. This issues will be addressed shortly and you will also be able to change the uid/gid of the webserver in the next devilbox release.

#### 4.2 PHP / HHVM

##### 4.2.1 Xdebug

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| PHP_XDEBUG_ENABLE         | `1` | Enable Xdebug for PHP |
| PHP_XDEBUG_REMOTE_PORT    | `9000` | PHP Xdebugs remote port - where your editor/IDE is receiving Xdebug data |
| PHP_XDEBUG_REMOTE_HOST    | `192.168.0.215` | PHP Xdebugs remote IP address - where your editor/IDE is listening for Xdebug connections (Make sure to change this to the IP of your Docker Host |

Xdebug is turned on by default and also using the official Xdebug default port. The only thing you will have to adjust is the Xdebug remote host address so that your editor/IDE can actually receive Xdebug data.

##### 4.2.2 php.ini

`php.ini` settings can be configured for each PHP/HHVM version separately. Container-based configuration is done inside the `./cfg/` directory.

```shell
$ ls -l ./cfg/ | grep -E 'php|hhvm'
drwxrwxr-x 2 cytopia 4096 Jun 21 08:47 hhvm-latest/
drwxrwxr-x 2 cytopia 4096 Jun 21 08:43 php-fpm-5.4/
drwxrwxr-x 2 cytopia 4096 Jun 21 08:43 php-fpm-5.5/
drwxrwxr-x 2 cytopia 4096 Jun 21 08:43 php-fpm-5.6/
drwxrwxr-x 2 cytopia 4096 Jun 21 08:44 php-fpm-7.0/
drwxrwxr-x 2 cytopia 4096 Jun 21 08:44 php-fpm-7.1/
drwxrwxr-x 2 cytopia 4096 Jun 21 08:44 php-fpm-7.2/
```

Each of the above folders will hold an example configuration file named `devilbox-custom.ini-example` which shows some example settings but will **not have** any effect yet. Only files ending by **`.ini`** will be sourced and applied, so you must copy it (or create a new file) to something that ends by `*.ini`.

In order to edit settings for PHP 5.6, go into that folder, copy the example file and adjust ist:

```shell
# Copy to file ending by *.ini
$ cd cfg/php-fpm-5.6
$ cp devilbox-custom.ini-example devilbox-custom.ini

# Edit settings
$ vi devilbox-custom.ini
```

Change will take effect after restarting the devilbox.

##### 4.2.3 HHVM

HHVM can just be configured as all other PHP versions. However it has a special option to change between **PHP-5.6** mode and **PHP-7** mode. This example is addressed in `cfg/hhvm-latest/devilbox.ini-example`.

```shell
$ cat cfg/hhvm-latest/devilbox.ini-example
```
```ini
; Use PHP 5.6 or PHP 7 mode for HHVM
; PHP 5.6 Mode
;hhvm.php7.all = 0
; PHP 7 Mode (default)
;hhvm.php7.all = 1
```

By default, HHVM is using **PHP-7** mode, you can change this setting to **PHP-5.6** by enabling `hhvm.php7.all = 0`.

**Note:** You must then also copy the file to something that ends by `*.ini`.

#### 4.3 Webserver

##### 4.3.1 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_HTTPD           | `80`| Host computer listening port for the webserver (Apache or Nginx) |

By default the webserver will listen on port 80 (on your Host computer). You can change this to any other port (in case port 80 is already taken).

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

#### 4.4 MySQL

##### 4.4.1 Root password

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| MYSQL_ROOT_PASSWORD | `` | Root user password for MySQL |

If you start a MySQL container for the first time, it will setup MySQL itself with the specified password. If you do change the root password to something else, make sure to also set it accordingly in `.env`, otherwise the devilbox will not be able to connect to MySQL and will not be able to display information inside the bundled intranet.

##### 4.4.2 General Log

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| MYSQL_GENERAL_LOG | `1` | Turn on or off MySQL general query log |

https://dev.mysql.com/doc/refman/5.7/en/query-log.html:
> The general query log is a general record of what mysqld is doing. The server writes information to this log when clients connect or disconnect, and it logs each SQL statement received from clients. The general query log can be very useful when you suspect an error in a client and want to know exactly what the client sent to mysqld.

This logging behavior is turned on by default in order to provide all information during development. Set it to `0` to turn it off.

##### 4.4.3 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_MYSQL           | `3306`| Host computer listening port for the MySQL server (MySQL or MariaDB) |

By default the MySQL server will listen on port 3306 (on your Host computer). You can change this to any other port (in case port 3306 is already taken).

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

##### 4.4.4 Data path

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PATH_MYSQL_DATADIR   | `./data/mysql`   | Can be absolute or relative path. A relative path starts inside the devilbox git directory. |

This is the file system path on your host computer which will hold the MySQL data.

**Note:** A sub directory will be created inside this path for each MySQL version. This separation is there to make sure that higher versions do not upgrade the database irrevocably. (e.g.: MySQL 8.0 can read data from MySQL 5.5, but not the other way round).

The automatic folder structure will look something like this:
```shell
$ ls -l ./data/mysql/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mariadb-10.0/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mariadb-10.1/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mariadb-10.2/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mariadb-10.3/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mysqp-5.5/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mysqp-5.6/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mysqp-5.7/
drwxrwxr-x 6 48 48 4096 Jun 21 08:47 mysqp-8.0/
```

##### 4.4.5 my.cnf

`my.cnf` settings can be configured for each MySQL/MariaDB version separately. Container-based configuration is done inside the `./cfg/` directory.

```shell
$ ls -l ./cfg/ | grep -E 'mysql|maria'
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mariadb-10.0/
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mariadb-10.1/
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mariadb-10.2/
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mariadb-10.3/
drwxrwxr-x 2 cytopia 4096 Jun 13 13:18 mysql-5.5/
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mysql-5.6/
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mysql-5.7/
drwxrwxr-x 2 cytopia 4096 Jun  1 08:44 mysql-8.0/
```

Each of the above folders will hold an example configuration file named `devilbox-custom.cnf-example` which shows some example settings but will **not have** any effect yet. Only files ending by **`.cnf`** will be sourced and applied, so you must copy it (or create a new file) to something that ends by `*.cnf`.

In order to edit settings for MySQL 5.5, go into that folder, copy the example file and adjust ist:

```shell
# Copy to file ending by *.ini
$ cd cfg/mysql-5.5
$ cp devilbox-custom.cnf-example devilbox-custom.cnf

# Edit settings
$ vi devilbox-custom.cnf
```

Change will take effect after restarting the devilbox.


#### 4.5 PostgreSQL
##### 4.5.1 Root user
##### 4.5.2 Root password
##### 4.5.3 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_PGSQL           | `5432`| Host computer listening port for the PostgreSQL server |

By default the PostgreSQL server will listen on port 5432 (on your Host computer). You can change this to any other port (in case port 5432 is already taken).

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

##### 4.5.4 Data path

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PATH_PGSQL_DATADIR   | `./data/pgsql`   | Can be absolute or relative path. A relative path starts inside the devilbox git directory. |

#### 4.6 Redis

##### 4.6.1 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_REDIS           | `6379`| Host computer listening port for the Redis server |

By default the Redis server will listen on port 6379 (on your Host computer). You can change this to any other port (in case port 6379 is already taken).

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

#### 4.7 Memcached

##### 4.7.1 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_MEMCD           | `11211`| Host computer listening port for the Memcached server |

By default the Memcached server will listen on port 11211 (on your Host computer). You can change this to any other port (in case port 11211 is already taken).

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

#### 4.8 MongoDB

##### 4.8.1 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_MONGO           | `27017`| Host computer listening port for the MongoDB server |

By default the Memcached server will listen on port 27017 (on your Host computer). You can change this to any other port (in case port 27017 is already taken).

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

##### 4.8.2 Data path

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PATH_MONGO_DATADIR   | `./data/mongo`   | Can be absolute or relative path. A relative path starts inside the devilbox git directory. |



#### 4.9 Bind
##### 4.9.1 Upstream resolver
##### 4.9.2 Host port

| `.env` file variable name | Default | Note |
|---------------------------|---------|------|
| HOST_PORT_BIND           | `1053`| Host computer listening port for the Bind DNS server |

By default the Bind DNS server will listen on port 1053 (on your Host computer). You can change this to any other port (in case port 1053 is already taken)

If you also want to change the listening address (default: 127.0.0.1) to something else, see above or search this document for `LOCAL_LISTEN_ADDRESS`.

## 5. Intranet settings
#### 5.1 DNS check timeout
## 6. Host computer
#### 6.1 Auto-DNS
#### 6.2 /etc/hosts/

